/**
 * Copyright (C) 2017 athinkthings
 *
 */
var KEY_ANNEX_TAG = "attAnnex"
var RE = {};

RE.currentSelection = {
    "startContainer": 0,
    "startOffset": 0,
    "endContainer": 0,
    "endOffset": 0};

RE.editor = document.getElementById('editor');

document.addEventListener("selectionchange", function() { RE.backuprange(); });

// Initializations
RE.textChange = function() {
    window.javaHandler.submitHtml(RE.getHtml());
    RE.undoStateChange();
}

RE.undoStateChange = function() {
    window.javaHandler.undoStatuChange((document.queryCommandEnabled("undo")?"1":"0")
        +";" + (document.queryCommandEnabled("redo")?"1":"0"));
}

RE.setHtml = function(contents) {
    RE.editor.innerHTML = decodeURIComponent(contents.replace(/\+/g, '%20'));
    RE.checkAnnex();
}

RE.getHtml = function() {
    return RE.editor.innerHTML;
}

RE.getText = function() {
    return RE.editor.innerText;
}

RE.setBaseTextColor = function(color) {
    RE.editor.style.color  = color;
}

RE.setBaseFontSize = function(size) {
    RE.editor.style.fontSize = size;
}

RE.setPadding = function(left, top, right, bottom) {
  RE.editor.style.paddingLeft = left;
  RE.editor.style.paddingTop = top;
  RE.editor.style.paddingRight = right;
  RE.editor.style.paddingBottom = bottom;
}

RE.setBackgroundColor = function(color) {
    document.body.style.backgroundColor = color;
}

RE.setBackgroundImage = function(image) {
    RE.editor.style.backgroundImage = image;
}

RE.setWidth = function(size) {
    RE.editor.style.minWidth = size;
}

RE.setHeight = function(size) {
    RE.editor.style.height = size;
}

RE.setTextAlign = function(align) {
    RE.editor.style.textAlign = align;
}

RE.setVerticalAlign = function(align) {
    RE.editor.style.verticalAlign = align;
}

RE.setPlaceholder = function(placeholder) {
    RE.editor.setAttribute("placeholder", placeholder);
}

RE.setInputEnabled = function(inputEnabled) {
    RE.editor.contentEditable = String(inputEnabled);
}

RE.undo = function() {
    document.execCommand('undo', false, null);
    RE.undoStateChange();
}

RE.redo = function() {
    document.execCommand('redo', false, null);
    RE.undoStateChange();
}

RE.setBold = function() {
    document.execCommand('bold', false, null);
    RE.undoStateChange();
}

RE.setItalic = function() {
    document.execCommand('italic', false, null);
    RE.undoStateChange();
}

RE.setSubscript = function() {
    document.execCommand('subscript', false, null);
    RE.undoStateChange();
}

RE.setSuperscript = function() {
    document.execCommand('superscript', false, null);
    RE.undoStateChange();
}

RE.setStrikeThrough = function() {
    document.execCommand('strikeThrough', false, null);
    RE.undoStateChange();
}

RE.setUnderline = function() {
    document.execCommand('underline', false, null);
    RE.undoStateChange();
}

RE.setBullets = function() {
    document.execCommand('insertUnorderedList', false, null);
    RE.undoStateChange();
}

RE.setNumbers = function() {
    document.execCommand('insertOrderedList', false, null);
    RE.undoStateChange();
}

RE.setTextColor = function(color) {
    //加入这个后插入的是<span style='color:rgb(*,*,*);'> 不加是 <font color='#8888'>
    //document.execCommand("styleWithCSS", null, true);
    document.execCommand('foreColor', false, color);
    //document.execCommand("styleWithCSS", null, false);
    RE.undoStateChange();
}

RE.setTextBackgroundColor = function(color) {
    //document.execCommand("styleWithCSS", null, true);
    document.execCommand('backColor', false, color);
    //document.execCommand("styleWithCSS", null, false);
    RE.undoStateChange();
}

RE.setFontSize = function(fontSize){
    document.execCommand("fontSize", false, fontSize);
    RE.undoStateChange();
}

RE.setHeading = function(heading) {
    document.execCommand('formatBlock', false, '<h'+heading+'>');
    RE.undoStateChange();
}

RE.setIndent = function() {
    document.execCommand('indent', false, null);
    RE.undoStateChange();
}

RE.setOutdent = function() {
    document.execCommand('outdent', false, null);
    RE.undoStateChange();
}

RE.setJustifyLeft = function() {
    document.execCommand('justifyLeft', false, null);
    RE.undoStateChange();
}

RE.setJustifyCenter = function() {
    document.execCommand('justifyCenter', false, null);
    RE.undoStateChange();
}


RE.setJustifyFull = function() {
    document.execCommand('justifyFull', false, null);
    RE.undoStateChange();
}
RE.setJustifyRight = function() {
    document.execCommand('justifyRight', false, null);
    RE.undoStateChange();
}

RE.setBlockquote = function() {
    var selectedElement = window.getSelection().focusNode.parentNode;
    if(selectedElement!=null && selectedElement.tagName=="BLOCKQUOTE"){
           document.execCommand('formatBlock', false, 'p');
    }else{
        document.execCommand('formatBlock', false, 'blockquote');
    }
    document.execCommand('insertHTML', false, "<br/>");
    RE.undoStateChange();
}

RE.insertHTML = function(html) {
     RE.restorerange();
     document.execCommand('insertHTML', false, html);
     RE.textChange();
}

RE.insertHtmlSimple = function(html) {
    document.execCommand('insertHTML', false, html);
    RE.textChange();
}

RE.insertHorizontalRule = function(){
    document.execCommand('insertHorizontalRule', false, null);
    RE.undoStateChange();
}

RE.insertLink = function(url, title) {
    RE.restorerange();
    var sel = document.getSelection();
    if (sel.toString().length == 0) {
        document.execCommand("insertHTML",false,"<a href='"+url+"'>"+title+"</a>");
    } else if (sel.rangeCount) {
       var el = document.createElement("a");
       el.setAttribute("href", url);
       el.setAttribute("title", title);

       var range = sel.getRangeAt(0).cloneRange();
       range.surroundContents(el);
       sel.removeAllRanges();
       sel.addRange(range);
   }
    RE.textChange();
}

RE.setTodo = function() {
    var html = '<input type="checkbox" onclick="checkBoxClick(this);"/>&nbsp;';
    document.execCommand('insertHTML', false, html);
    RE.textChange();
}

function checkBoxClick(e){
    if (e.checked) e.setAttribute('checked', true);
    else e.removeAttribute('checked');
    RE.textChange();
}

RE.prepareInsert = function() {
    RE.backuprange();
}



RE.backuprange = function(){
    var selection = window.getSelection();
    if (selection.rangeCount > 0) {
      var range = selection.getRangeAt(0);
      RE.currentSelection = {
          "startContainer": range.startContainer,
          "startOffset": range.startOffset,
          "endContainer": range.endContainer,
          "endOffset": range.endOffset};
    }
}

RE.restorerange = function(){
    var selection = window.getSelection();
    selection.removeAllRanges();
    var range = document.createRange();
    range.setStart(RE.currentSelection.startContainer, RE.currentSelection.startOffset);
    range.setEnd(RE.currentSelection.endContainer, RE.currentSelection.endOffset);
    selection.addRange(range);
}

RE.enabledEditingItems = function(e) {
    RE.sendStatus();
}

RE.onBodyClick = function(e) {
    var k =e.srcElement;
    if(k.tagName=="IMG"){
        //所有图片都可点击打开
       //if(k.getAttribute("tag")== KEY_ANNEX_TAG){
          onclickImg(k,k.getAttribute("tag")== KEY_ANNEX_TAG);
          return;
      // }
    }
    if(k.tagName=="A"){
       window.javaHandler.onBodyClick("link",encodeURI(k.href),k.innerText);
       return false;
    }
    window.javaHandler.onBodyClick("body","","");
    RE.sendStatus();
}

RE.sendStatus = function() {
    var items = [];
    items.push("fontsize:"+document.queryCommandValue("fontSize"));
    items.push("forecolor:"+document.queryCommandValue("foreColor"));
    items.push("backcolor:"+document.queryCommandValue("backColor"));
    if (document.queryCommandState('bold')) {
        items.push('bold');
    }
    if (document.queryCommandState('italic')) {
        items.push('italic');
    }
    if (document.queryCommandState('subScript')) {
        items.push('subscript');
    }
    if (document.queryCommandState('superScript')) {
        items.push('superscript');
    }
    if (document.queryCommandState('strikeThrough')) {
        items.push('strikeThrough');
    }
    if (document.queryCommandState('underline')) {
        items.push('underline');
    }
    if (document.queryCommandState('insertOrderedList')) {
        items.push('orderedList');
    }
    if (document.queryCommandState('insertUnorderedList')) {
        items.push('unorderedList');
    }
    if (document.queryCommandState('justifyCenter')) {
        items.push('justifyCenter');
    }
    if (document.queryCommandState('justifyFull')) {
        items.push('justifyFull');
    }
    if (document.queryCommandState('justifyLeft')) {
        items.push('justifyLeft');
    }
    if (document.queryCommandState('justifyRight')) {
        items.push('justifyRight');
    }
    if (document.queryCommandState('insertHorizontalRule')) {
        items.push('horizontalRule');
    }
    var formatBlock = document.queryCommandValue('formatBlock');
    if (formatBlock.length > 0) {
        items.push(formatBlock);
    }

    window.location.href = "re-state://" + encodeURI(items.join(';'));
}

RE.focus = function() {
    var range = document.createRange();
    range.selectNodeContents(RE.editor);
    range.collapse(false);
    var selection = window.getSelection();
    selection.removeAllRanges();
    selection.addRange(range);
    RE.editor.focus();
}


RE.blurFocus = function() {
    RE.editor.blur();
}

RE.removeFormat = function() {
    document.execCommand('removeFormat', false, null);
}

RE.lastWordRange = null;
function autoLink(){
    if (RE.lastWordRange == null) return;
    var keyword = RE.lastWordRange.startContainer.textContent;
    keyword = keyword.replace(/(\s*$)/g, "");
    var so = keyword.lastIndexOf(" ");
    if (so != -1) {
        keyword = keyword.substr(so + 1);
    }
    var defaultScheme = 'http://';
    var linkPattern = /^([A-Za-z][A-Za-z0-9+-.]*\:[\/\/]?|mailto:[A-Z0-9._%+-]+@)?(www\.)?(.+)$/i;

    var match = keyword.match(linkPattern);
    if (match && (match[1] || match[2])) {
        var link = match[1] ? keyword : defaultScheme + keyword;
        var node = document.createElement('a');
        node.href=link;
        node.innerHTML=keyword;
        node.target="_blank";
        RE.lastWordRange.setStart(RE.lastWordRange.startContainer, so + 1);
        RE.lastWordRange.deleteContents();
        RE.lastWordRange.insertNode(node);
        RE.lastWordRange = null;
    }
    RE.textChange();
}


function onclickImg(img,isAnnex){
    var src = img.getAttribute("src")
    if(src.length>1000) return;
    if(src.lastIndexOf("?")>3) src = src.substr(0,src.lastIndexOf("?"));
    window.javaHandler.onBodyClick("image",encodeURI(src),isAnnex?"1":"0");
}

RE.insertImage = function(url, alt) {
    var html = "<p><img class='img-responsive' onerror='attImgNoFind(this)' alt='" + alt + "' src='" + url
        + "?t="+  Math.random() + "' tag='"+KEY_ANNEX_TAG+"'/></p><br/>";
    RE.insertHTML(html);
}
RE.insertImageBr2 = function(url, alt) {
    var html = "<p><img class='img-responsive' onerror='attImgNoFind(this)' alt='" + alt + "' src='" + url
        + "?t="+  Math.random() + "' tag='"+KEY_ANNEX_TAG+"'/></p><br/><br/>";
    RE.insertHTML(html);
}

RE.updateImage = function(url){
    var img = findImg(url);
    if(img==null) return;
    img.attr('src',url+"?t="+Math.random());
    RE.textChange();
}

//在1.00版有的地方未加 onerror，所以检测一下
RE.checkAnnex = function(){
    var isEdit = false;
    $("img,[tag='"+KEY_ANNEX_TAG+"']").each(function () {
        if(typeof($(this).attr("onerror"))=="undefined"){
            $(this).attr("onerror","attImgNoFind(this)");
            isEdit = true;
        }
    }); 
    if(isEdit) RE.textChange();
}

findImg = function(url){
    var imgs = $("[src^='"+url+"']");
    if (imgs == null || imgs.length<1) return null;
    return imgs.eq(0);
}

RE.deleteImage=function(url){
    var img = findImg(url);
    if(img==null) return;
    img.remove();
    RE.textChange();
}


// Event Listeners
RE.editor.addEventListener("input", RE.textChange);
RE.editor.addEventListener("keyup", function(e) {
    var KEY_LEFT = 37, KEY_RIGHT = 39,KEY_DOWN = 38,KEY_UP = 40,ENTER=13,SPACE=32,SPACE1=229,DEL=46,BACKDEL=8;
    var wh = e.which;
    if (wh == KEY_LEFT || wh == KEY_RIGHT||wh == KEY_DOWN|| wh == KEY_UP|| wh == DEL|| wh == BACKDEL) {
        RE.enabledEditingItems(e);
    }
    if (wh == ENTER || wh == SPACE || wh == SPACE1) {
         autoLink();
     }
});
RE.editor.addEventListener("keydown", function (e) {
    ENTER = 13, SPACE = 32,SPACE1=229;
    if (e.which == ENTER || e.which == SPACE||  e.which == SPACE1) {
        RE.lastWordRange = window.getSelection().getRangeAt(0);
    }
});
RE.editor.addEventListener("click", RE.onBodyClick);

function attImgNoFind(_this){
    var src = _this.getAttribute('src');
    if(src.lastIndexOf("?")>3) src = src.substr(0,src.lastIndexOf("?"));
    window.javaHandler.onImageError(encodeURI(src));
    _this.onerror=null;
}

function myPaste(event){
    window.javaHandler.onPaste();
    return true;
}
window.onload=function() {
    document.getElementById('editor').onpaste=function(){return myPaste(event);};
}